
{*******************************************************}
{                                                       }
{       Delphi VCL Extension Library                    }
{                                                       }
{       Copyright (c) 1998 Stephan Schneider            }
{                                                       }
{*******************************************************}

unit IniFiles32;

{$B-}

interface

uses Classes;

type

{ TIniFile32 class }

  TIniFile32 = class(TObject)
  private
    FFileName: String;
    FFileBuffer: TStringList;
  protected
    function GetSectionIndex(const Section: String): Integer;
    procedure LoadFromFile;
    procedure SaveToFile;
  public
    constructor Create(const FileName: String);
    destructor Destroy; override;
    procedure DeleteKey(const Section, Ident: String);
    procedure EraseSection(const Section: String);
    function ReadBool(const Section, Ident: String; Default: Boolean): Boolean;
    function ReadInteger(const Section, Ident: String; Default: Longint): Longint;
    procedure ReadSection(const Section: String; Strings: TStrings);
    procedure ReadSections(Strings: TStrings);
    procedure ReadSectionValues(const Section: String; Strings: TStrings);
    function ReadString(const Section, Ident: String; Default: String): String;
    procedure WriteBool(const Section, Ident: String; Value: Boolean);
    procedure WriteInteger(const Section, Ident: String; Value: Longint);
    procedure WriteString(const Section, Ident: String; Value: String);
    property FileName: String read FFileName;
  end;

implementation

uses SysUtils;

resourcestring

{ Private Assertions }

  SStringsUnassignedError = 'Param Strings must be assigned';

const
  Brackets: array[0..1] of Char = ('[', ']');

function AnsiPos(const Substr, S: String): Integer;
begin
  Result := System.Pos(Substr, S);
end;

{ TIniFile32 }

constructor TIniFile32.Create(const FileName: String);
begin
  FFileName := FileName;
  FFileBuffer := TStringList.Create;
  if FileExists(FileName) then LoadFromFile;
end;

destructor TIniFile32.Destroy;
begin
  FFileBuffer.Free;
  Finalize(FFileName);
end;

procedure TIniFile32.DeleteKey(const Section, Ident: String);
var
  I: Integer;
begin
  I := GetSectionIndex(Section);
  if I <> -1 then
  begin
    Inc(I);
    while (I < FFileBuffer.Count) and (AnsiPos(Brackets[0], FFileBuffer[I]) = 0) and
      (FFileBuffer.Names[I] <> Ident) do Inc(I);
    //  Ident does exists  
    if not (I >= FFileBuffer.Count) and not (AnsiPos(Brackets[0], FFileBuffer[I]) <> 0) then
    begin
      FFileBuffer.Delete(I);
      SaveToFile;
    end;  
  end;
end;

procedure TIniFile32.EraseSection(const Section: String);
var
  I: Integer;
begin
  I := GetSectionIndex(Section);
  if I <> -1 then
  begin
    // Delete Section-Header
    FFileBuffer.Delete(I);
    // Delete Section-Items
    while (I < FFileBuffer.Count) and (AnsiPos(Brackets[0], FFileBuffer[I]) = 0) do
      FFileBuffer.Delete(I);
    if I > 0 then FFileBuffer.Insert(I, EmptyStr);  
    SaveToFile;
  end;
end;

function TIniFile32.GetSectionIndex(const Section: String): Integer;
begin
  Result := FFileBuffer.IndexOf(Brackets[0] + Section + Brackets[1]);
end;

procedure TIniFile32.LoadFromFile;
begin
  FFileBuffer.LoadFromFile(FFileName);
end;

function TIniFile32.ReadBool(const Section, Ident: String; Default: Boolean): Boolean;
begin
  Result := ReadInteger(Section, Ident, Ord(Default)) <> 0;
end;

function TIniFile32.ReadInteger(const Section, Ident: String; Default: Longint): Longint;
var
  IntStr: string;
begin
  IntStr := ReadString(Section, Ident, '');
  if (Length(IntStr) > 2) and (IntStr[1] = '0') and ((IntStr[2] = 'X') or (IntStr[2] = 'x')) then
    IntStr := '$' + Copy(IntStr, 3, Maxint);
  Result := StrToIntDef(IntStr, Default);
end;

procedure TIniFile32.ReadSection(const Section: String; Strings: TStrings);
var
  I: Integer;
begin
  Assert(Assigned(Strings), SStringsUnassignedError);
  Strings.BeginUpdate;
  try
    Strings.Clear;
    if FFileBuffer.Count > 0 then
    begin
      I := GetSectionIndex(Section);
      if I <> -1 then
      begin
        Inc(I);
        while (I < FFileBuffer.Count) and (AnsiPos(Brackets[0], FFileBuffer[I]) = 0) do
        begin
          if (FFileBuffer.Names[I] <> EmptyStr) then Strings.Add(FFileBuffer.Names[I]);
          Inc(I);
        end;
      end;
    end;
  finally
    Strings.EndUpdate;
  end;
end;

procedure TIniFile32.ReadSections(Strings: TStrings);
var
  I: Integer;
  Section: String;
begin
  Assert(Assigned(Strings), SStringsUnassignedError);
  Strings.BeginUpdate;
  try
    Strings.Clear;
    if FFileBuffer.Count > 0 then
    begin
      I := 0;
      while (I < FFileBuffer.Count) do
      begin
        if (AnsiPos(Brackets[0], FFileBuffer[I]) <> 0) and (AnsiPos(Brackets[1], FFileBuffer[I]) <> 0) then
        begin
          Section := FFileBuffer[I];
          System.Delete(Section, 1, AnsiPos(Brackets[0], Section));
          System.Delete(Section, AnsiPos(Brackets[1], Section), MaxInt);
          Strings.Add(Section);
        end;
        Inc(I);
      end;  
    end;
  finally
    Strings.EndUpdate;
  end;
end;

procedure TIniFile32.ReadSectionValues(const Section: String; Strings: TStrings);
var
  KeyList: TStringList;
  I: Integer;
begin
  Assert(Assigned(Strings), SStringsUnassignedError);
  KeyList := TStringList.Create;
  try
    ReadSection(Section, KeyList);
    Strings.BeginUpdate;
    try
      Strings.Clear;
      for I := 0 to KeyList.Count - 1 do
        Strings.Values[KeyList[I]] := ReadString(Section, KeyList[I], ''); 
    finally
      Strings.EndUpdate;
    end;
  finally
    KeyList.Free;
  end;
end;

function TIniFile32.ReadString(const Section, Ident: String; Default: String): String;
var
  I: Integer;
begin
  Result := Default;
  if FFileBuffer.Count > 0 then
  begin
    I := GetSectionIndex(Section);
    if I <> -1 then
    begin
      Inc(I);
      while (I < FFileBuffer.Count) and (AnsiPos(Brackets[0], FFileBuffer[I]) = 0) do
      begin
        if (FFileBuffer.Names[I] = Ident) then
        begin
          Result := System.Copy(FFileBuffer[I], AnsiPos('=', FFileBuffer[I]) + 1, Maxint);
          Exit;
        end;
        Inc(I);
      end;
    end;
  end;
end;

procedure TIniFile32.SaveToFile;
begin
  FFileBuffer.SaveToFile(FFileName);
end;

procedure TIniFile32.WriteBool(const Section, Ident: String; Value: Boolean);
const
  Values: array[Boolean] of string = ('0', '1');
begin
  WriteString(Section, Ident, Values[Value]);
end;

procedure TIniFile32.WriteInteger(const Section, Ident: String; Value: Longint);
begin
  WriteString(Section, Ident, IntToStr(Value));
end;

procedure TIniFile32.WriteString(const Section, Ident: String; Value: String);
var
  I: Integer;
begin
  I := GetSectionIndex(Section);
  // Section exists
  if I <> -1 then
  begin
    Inc(I);
    while (I < FFileBuffer.Count) and (AnsiPos(Brackets[0], FFileBuffer[I]) = 0) and
      (FFileBuffer.Names[I] <> Ident) do Inc(I);
    // End of File or Ident doesn't exists in the Section  
    if (I >= FFileBuffer.Count) or (AnsiPos(Brackets[0], FFileBuffer[I]) <> 0) then
      FFileBuffer.Insert(I, Ident + '=' + Value)
    // Ident does exists in the section  
    else FFileBuffer[I] := Ident + '=' + Value;
  end
  // Section doesn't exists, so add new Section with Ident=Value
  else
  begin
    FFileBuffer.Add(EmptyStr);
    FFileBuffer.Add(Brackets[0] + Section + Brackets[1]);
    FFileBuffer.Add(Ident + '=' + Value);
  end;
  SaveToFile;
end;

end.
